/*-!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!
! Filename: ram2hex.c      Function: convert DIY-Calc RAM file into Intel  !
! Part of :                  HEX format                                    !
! Module  :                                                                !
!                                                                          !
!                                                                          !
!                                                                          !
!   Vers     Date    Auth  Changes                                         !
! ------------------------------------------------------------------------ !
!   1.0    12.Sep.06  JH   New.                                            !
!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-*/


#define VER "1.0"


#ifdef WIN32
#include <windows.h>
#endif

#ifdef __MSDOS__
#include <dir.h>             /* MAXPATH */
#endif

#ifdef __linux__
#include <limits.h>          /* PATH_MAX */
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>


/******************************  Constants  *******************************/
#define OK        0
#define BUFLEN  200

#ifdef __linux__
#define PROGNAME "ram2hex"
#define MAX_PATH PATH_MAX
#elif defined WIN32
#define PROGNAME "ram2hex.exe"
#elif defined __MSDOS__
#define PROGNAME "RAM2HEX.EXE"
#define MAX_PATH MAXPATH
#else
#error "unsupported platform"
#endif


/*******************************  Macros  *********************************/
#ifndef HIBYTE
#define HIBYTE(x) (((x)>>8)&0xff)
#endif
#ifndef LOBYTE
#define LOBYTE(x) ((x)&0xff)
#endif


/**************************  Function Prototypes  *************************/
int atohex (char *s);             /* convert string to HEX number         */


/*******************************  Globals  ********************************/
enum REC_TYPE
{
  REC_DATA,
  REC_END
};


/* main */
/**************************************************************************/
int main (int argc, char *argv[])
/***************************************************************************
* globals  :                                                               *
* called by:                                                               *
* calls    :                                                               *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* 0 .. OK                                                                  *
***************************************************************************/
{
  char hexname[MAX_PATH], ramname[MAX_PATH];
  FILE *hex, *ram;
  char buf[BUFLEN], tmp[12];
  char *p;
  unsigned int linecnt, bytecount, address, type, i, j;
  unsigned char data[BUFLEN/2], sum, checksum;

  if (argc == 1)
  {             
    fprintf (stderr, "usage: %s ramfile\n", PROGNAME );
    return 0;
  }
  printf ("\n%s V%s, %s\n\n", PROGNAME, VER, __DATE__);
  /* assemble file names */
  strcpy (ramname, argv[1]);
  p = strrchr (ramname, '.');
  if (p)
    *p = '\0';
  strcat (ramname, ".ram");

  strcpy (hexname, argv[1]);
  p = strchr (hexname, '.');
  if (p == NULL)
    strcat (hexname, ".hex");

  /* open the files */
  ram = fopen (ramname, "rt");
  if (ram == NULL)
  {
    perror (ramname);
    return 1;
  }

  hex = fopen (hexname, "wb");
  if (hex == NULL)
  {
    fclose (ram);
    perror (hexname);
    return 1;
  }

  printf ("Processing file \"%s\" ...    ", ramname);

  /* we now have opened the files; lets do stuff */
  linecnt = 8;

  /* skip the .ram file header */
  fgets (buf, BUFLEN, ram); /* # FILE TYPE: ... */
  fgets (buf, BUFLEN, ram); /* # GENERATED: ... */
  fgets (buf, BUFLEN, ram); /* # DATE-TIME: ... */
  fgets (buf, BUFLEN, ram); /* # SOURCEWAS: ... */
  fgets (buf, BUFLEN, ram); /* empty line */
  fgets (buf, BUFLEN, ram); /* # STARTADDR: ... */
  fgets (buf, BUFLEN, ram); /* # FINALADDR: ... */
  fgets (buf, BUFLEN, ram); /* empty line */

  bytecount = 16;
  type = REC_DATA;
  while (fgets (buf, BUFLEN, ram))
  {
    linecnt ++;
    printf ("\b\b\b\b%4d", linecnt);

    /* read the .ram file */
    tmp[0] = buf[1];
    tmp[1] = buf[2];
    tmp[2] = buf[3];
    tmp[3] = buf[4];
    tmp[4] = '\0';
    address = atohex (tmp);

    sum = HIBYTE(address) + LOBYTE(address) + type;
    for (i=0; i<bytecount; i++)
    {
      j = i * 3 + 6;
      tmp[0] = buf[j];
      j ++;
      tmp[1] = buf[j];
      tmp[2] = '\0';
      if (strcmp (tmp, "--") == 0)
        break;
      data[i] = atohex (tmp);
      sum += data[i];
    }
    sum += i;
    fprintf (hex, ":%2.2X%4.4X%2.2X", i, address, type);
    for (j=0; j<i; j++)
      fprintf (hex, "%2.2X", data[j]);
    sum = (-sum) & 0xff;
    fprintf (hex, "%2.2X\r\n", sum);
  }
  
  /* write end record */
  fprintf (hex, ":00000001FF\r\n");

  printf ("\b\b\b\b%4d lines done.\n", linecnt);
  fclose (hex);
  fclose (ram);

  return 0;
}

/* atohex */
/**************************************************************************/
int atohex (char *s)              /* convert string to HEX number         */
/***************************************************************************
* globals     :                                                            *
* called from : main()                                                     *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Convert string to HEX number                                             *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* Zahl (Integer)                                                           *
***************************************************************************/
{
   int i, digit, number;

   i = 0;
   number = 0;
   while (s[i]) {
     switch (toupper(s[i])) {
       case '0': digit =  0; break;
       case '1': digit =  1; break;
       case '2': digit =  2; break;
       case '3': digit =  3; break;
       case '4': digit =  4; break;
       case '5': digit =  5; break;
       case '6': digit =  6; break;
       case '7': digit =  7; break;
       case '8': digit =  8; break;
       case '9': digit =  9; break;
       case 'A': digit = 10; break;
       case 'B': digit = 11; break;
       case 'C': digit = 12; break;
       case 'D': digit = 13; break;
       case 'E': digit = 14; break;
       case 'F': digit = 15; break;
     }
     number = (number << 4) + digit;
     i++;
   }
/* printf("Number = %i, 0x%X\n", number, number);*/
   return (number);
}
